/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is locale4j.
 *
 * The Initial Developer of the Original Code is
 * the Vermont Department of Taxes.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Tom Cort <tom.cort@state.vt.us>
 *
 * ***** END LICENSE BLOCK ***** */

package net.sf.locale4j.format;

import net.sf.locale4j.Constants;

import net.sf.locale4j.storage.LocaleStore;
import net.sf.locale4j.storage.HashTableStore;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.xmlbeans.XmlAnySimpleType;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlOptions;

import org.lisa.tmx20.TmxDocument;
import org.lisa.tmx20.BodyDocument.Body;
import org.lisa.tmx20.Segtypes;
import org.lisa.tmx20.SegDocument.Seg;
import org.lisa.tmx20.TmxDocument.Tmx;
import org.lisa.tmx20.HeaderDocument.Header;
import org.lisa.tmx20.TmxDocument.Tmx.Version;
import org.lisa.tmx20.TuDocument.Tu;
import org.lisa.tmx20.TuvDocument.Tuv;

/**
 * Translation Memory eXchange Format.
 */
public class Tmx20 implements LocaleReader, LocaleWriter {

    /**
     * Reads a stream and loads the contents into a LocaleStore.
     * @param input the stream to read from.
     * @return the data read by this method.
     */
    public final LocaleStore read(InputStream input) {
        HashTableStore store = new HashTableStore();
        return this.read(input, store);
    }

    /**
     * Reads a stream and adds the contents to the given LocaleStore.
     * @param input the stream to read from.
     * @param store the LocaleStore to add to.
     * @return the data read by this method.
     */
    public final LocaleStore read(InputStream input, LocaleStore store) {

        try {
            StringBuffer buffer = new StringBuffer();
            InputStreamReader isr = new InputStreamReader(input, "UTF8"); 
            Reader in = new BufferedReader(isr);
            int ch;
            while ((ch = in.read()) > -1) {
                buffer.append((char) ch);
            }
            in.close();

            Map<String, String> nsSubst = new HashMap<String, String>();
            nsSubst.put("", "http://www.lisa.org/tmx20");
            XmlOptions parseOptions = new XmlOptions();
            parseOptions.setLoadSubstituteNamespaces(nsSubst);

            TmxDocument tmxDoc = TmxDocument.Factory.parse(buffer.toString(), parseOptions);

            if (!tmxDoc.validate()) {
                return null;
            }

            Tmx   tmx = tmxDoc.getTmx();
            Body body = tmx.getBody();
            Tu[]   tu = body.getTuArray();

            for (int i = 0; i < tu.length; i++) {
                String key = tu[i].getTuid().getStringValue();
                Tuv[]  tuv = tu[i].getTuvArray();
                for (int j = 0; j < tuv.length; j++) {
                    String lang = tuv[j].getLang();
                    Seg     seg = tuv[j].getSeg();

                    // <tmx:seg/> confuses XMLBeans, so we have to force it to provide us with a getTextValue()
                    XmlCursor cursor = seg.newCursor();
                    String value = cursor.getTextValue();
                    cursor.dispose();

                    store.put(new Locale(lang), key, value);
                }
            }
        } catch (Exception e) {
            return null;
        }

        return store;
    }

    /**
     * Writes data to a file.
     * @param output the stream to write to.
     * @param store the data to write to the file.
     */
    public void write(OutputStream output, LocaleStore store) {
        Map<String, String> nsSubst = new HashMap<String, String>();
        nsSubst.put("", "http://www.lisa.org/tmx20");
        XmlOptions parseOptions = new XmlOptions();
        parseOptions.setLoadSubstituteNamespaces(nsSubst);

        TmxDocument   tmxDoc = TmxDocument.Factory.newInstance(parseOptions);
        List<Locale> locales = store.getLocales();
        List<String>    keys = store.getKeys(locales.get(0));

        // <tmx:tmx>
        Tmx tmx = tmxDoc.addNewTmx();
        tmx.setVersion(Version.X_2_0);

            // <tmx:header>
            Header header = tmx.addNewHeader();

                // adminlang="en"
                XmlAnySimpleType adminLang = header.addNewAdminlang();
                adminLang.setStringValue("en");

                // creationtool="locale4j"
                XmlAnySimpleType creationTool = header.addNewCreationtool();
                creationTool.setStringValue(Constants.LIBRARY_NAME);

                // creationtoolversion="1.0"
                XmlAnySimpleType creationToolVersion = header.addNewCreationtoolversion();
                creationToolVersion.setStringValue(Constants.LIBRARY_VERSION);

                // o-tmf="unknown"
                XmlAnySimpleType oTmf = header.addNewOTmf();
                oTmf.setStringValue("unknown");

                // segtype="block"
                header.setSegtype(Segtypes.BLOCK);

                // srclang="*all*"
                XmlAnySimpleType srcLang = header.addNewSrclang();
                srcLang.setStringValue("*all*");

            // <tmx:body>
            Body body = tmx.addNewBody();

                for (int i = 0; i < keys.size(); i++) {
                    // <tmx:tu tuid="key">
                    Tu tu = body.addNewTu();
                    XmlAnySimpleType tuid = tu.addNewTuid();
                    tuid.setStringValue(keys.get(i));

                    for (int j = 0; j < locales.size(); j++) {
                        Tuv tuv = tu.addNewTuv(); // <tmx:tuv xml:lang="xx">
                        tuv.setLang(locales.get(j).getLanguage());

                            // <tmx:seg>
                            Seg seg = tuv.addNewSeg();
                            XmlCursor cursor = seg.newCursor();
                            cursor.setTextValue(store.get(locales.get(j), keys.get(i)));
                            cursor.dispose();
                            // </tmx:seg>
                    }

                    // </tmx:tu>
                }
            // </tmx:body>
        // </tmx:tmx>

        try {
            tmxDoc.save(output);
        } catch (IOException e) {
            return;
        }
    }
}

