/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is locale4j.
 *
 * The Initial Developer of the Original Code is
 * the Vermont Department of Taxes.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Tom Cort <tom.cort@state.vt.us>
 *   Matt Gagne <matt.gagne@state.vt.us>
 *
 * ***** END LICENSE BLOCK ***** */

package net.sf.locale4j.storage;

import java.util.ArrayList;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;

/**
 * In memory storage of locale data using HashTables.
 */
public class TreeMapStore implements LocaleStore {

    /**
     * List of Hash Tables to store parsed values for each locale.
     */
    private TreeMap < String, TreeMap < String, String > > table;

    /**
     * @return the table
     */
    public TreeMap < String, TreeMap < String, String > > getTable() {
        return table;
    }

    /**
     * @param table the table to set
     */
    public void setTable(TreeMap < String, TreeMap < String, String > > table) {
        this.table = table;
    }

    /**
     * Default Constructor.
     */
    public TreeMapStore() {
        table = new TreeMap < String, TreeMap < String, String > > ();
    }

    /**
     * Fetches a string in the language specified by the locale.
     * If the string does not exist, "???" is returned.
     * @param locale the locale for which the String is localized.
     * @param key the key used to reference the stored String.
     * @return the localized string.
     */
    public String get(Locale locale, String key) {
        if (locale == null || key == null || key.length() == 0) {
            return "???";
        }

        String lang = locale.getLanguage();

        if (table.containsKey(lang) && table.get(lang).containsKey(key)) {
            return table.get(lang).get(key);
        } else {
            return "???";
        }
    }

    /**
     * Stores a string in the language specified by the locale.
     * @param locale the locale for which the String is localized.
     * @param key the key used to reference the stored String.
     * @param text the String to store.
     */
    public void put(Locale locale, String key, String text) {
        if (locale == null || key == null || key.length() == 0 || text == null || text.length() == 0) {
            return;
        }

        String lang = locale.getLanguage();

        if (!table.containsKey(lang)) {
            // if a table for this language doesn't exist, create one.
            TreeMap < String, String > newLocaleTable = new TreeMap < String, String > ();

            // add key/value to new hashTable for this lang.
            newLocaleTable.put(key, text);

            // add the new locale table to the main table.
            table.put(lang, newLocaleTable);
        } else if (table.get(lang).containsKey(key)) {
            // if the key currently exists, remove it and add the new value
            table.get(lang).remove(key);
            table.get(lang).put(key, text);
        } else {
            //put this key/text pair in the appropriate HashTable
            table.get(lang).put(key, text);
        }
    }

    /**
     * Fetch a list of locales for which translations exist.
     * @return a list of locales.
     */
    public List < Locale > getLocales() {
        ArrayList < Locale > locales = new ArrayList < Locale > ();
        Set < String >          keys = table.keySet();
        Iterator < String >      itr = keys.iterator();

        while (itr.hasNext()) {
            locales.add(new Locale(itr.next()));
        }

        return locales;
    }

    /**
     * Fetch a list of keys for the given locale.
     * @param locale the locale of the Strings the keys point to.
     * @return a list of keys
     */
    public List < String > getKeys(Locale locale) {
        if (locale == null) {
            return new ArrayList < String > ();
        }

        String lang = locale.getLanguage();

        if (table.get(lang) == null) {
            return new ArrayList<String>();
        }

        Set < String >         keys = table.get(lang).keySet();
        ArrayList < String >   rset = new ArrayList < String > ();
        Iterator < String >     itr = keys.iterator();

        while (itr.hasNext()) {
            rset.add(itr.next());
        }

        return rset;
    }
}


